import { FC, useEffect, useState } from "react";
import {
  HubConnection,
  HubConnectionBuilder,
  LogLevel,
} from "@microsoft/signalr";
import { getApiBaseUrl } from "@src/infrastructure/apis/utilities";
import { useAppDispatch } from "@src/infrastructure/store";
import { todosApi } from "@src/infrastructure/apis/todosApi";

export const SocketProvider: FC = (props) => {
  const [conn, setConn] = useState<HubConnection>();
  const dispatch = useAppDispatch();

  useEffect(() => {
    const connection = new HubConnectionBuilder()
      .withUrl(`${getApiBaseUrl()}/hubs/todo`, {
        withCredentials: true,
      })
      .withAutomaticReconnect()
      .configureLogging(LogLevel.Information)
      .build();

    connection.on("todoCreated", (todoCreated) => {
      const { todoId } = JSON.parse(todoCreated) as { todoId: string };
      if (todoId) {
        dispatch(todosApi.endpoints.getTodoById.initiate(todoId));
        // dispatch(todosApi.util.invalidateTags([{ type: "todo", id: "LIST" }]));
      }
    });

    connection.on("todoUpdated", (todoUpdated) => {
      const { todoId } = JSON.parse(todoUpdated) as { todoId: string };
      if (todoId) {
        dispatch(todosApi.util.invalidateTags([{ type: "todo", id: todoId }]));
        // dispatch(todosApi.util.invalidateTags([{ type: "todo", id: "LIST" }]));
      }
    });

    connection
      .start()
      .then(() => {
        setConn(connection);
      })
      .catch((e) => {
        window.location.href = `${getApiBaseUrl()}/api/auth/login?returnUrl=${
          window.location.href
        }`;
      });
  }, []);

  return <>{props.children}</>;
};

export default SocketProvider;
